$title  GTAP Aggregation Routine -- Aggregates from any dimension dataset.

$ontext

-----------------------------------------------------------------------------------
Thomas F. Rutherford
Department of Economics

version April, 1999

Research Supported by US Department of Energy
-----------------------------------------------------------------------------------

Notes:

This version of the code incorporates IEO baseline statistics when
the source dataset is GTAPDOE.


	- 30 year time horizon (through 2030), based on an extrapolation of
	  the 20 year DOE projection.

	- Inputs GDP growth rates and imputes AEEI by fuel and region
	  to match the DOE/EIA International Energy Outlook

-----------------------------------------------------------------------------------
Parameters added to the standard GTAP dataset include:

	ieoprice(sc,t)			Defines baseline crude oil price (1995=1)
	ieocrude(sc,r,t)		Defines baseline crude oil output (1995=1)
	ieocarbon(sc,ffuel,r,tt)	Defines baseline fuel use (1995=1)
	ieogdp(sc,t,r)			Defines baseline GDP (1995=1)
	kyoto(r)			Kyoto targets as percentage of 1990 emissions

	annexb(r)	Set defining members of Annex B
	oecd(r)		Set defining members of the OECD
	eu(*)		Set defining members of the European Union
-----------------------------------------------------------------------------------

$offtext

*       Define archive format here:

$setglobal dataset 01

*       Read the source sets (these were generated by gtapsets):

$include gtapaggr.set

*       Read the dataset name:

$include gtapaggr.inc

*       Read the target sets:

$include ..\defines\%dataset%.set

*       Declare the mappings so that we do domain checking on input:

set     mapr(rr,r), mapi(ii,i);

*       Read the mappings:

$include ..\defines\%dataset%.map

$call 'zip -j ..\data\%dataset% ..\defines\%dataset%.set >nul';
$call 'zip -j ..\data\%dataset% ..\defines\%dataset%.map >nul';
$call 'zip -j ..\data\%dataset% ..\defines\%source%.set >nul';
$if exist ..\defines\%source%.map $call 'zip -j ..\data\%dataset% ..\defines\%source%.map >nul';

SET     D Classes of demand for which import shares are differentiated / 
        G       Public, 
        C       Private Consumption,
        I       Intermediate input /;

ALIAS (I,J), (R,S), (D,DD), (JJ,II), (SS,RR);

*       Assign default mappings if they are not defined:

$if not defined MAPI MAPI(II,I)$SAMEAS(II,I)  = YES;
$if not defined MAPR MAPR(RR,R)$SAMEAS(RR,R)  = YES;

$if not declared FF   ALIAS (F,FF);
$if not declared MAPF SET MAPF(F,FF); MAPF(F,F)  = YES;

SET MAPJ(JJ,J); MAPJ(JJ,J) = MAPI(JJ,J);
SET MAPS(SS,S); MAPS(SS,S) = MAPR(SS,S);
SET MAPD(D,DD); MAPD(D,D)  = YES;

*       Read the source data:

parameter
        ty(ii,rr)       Output tax
        ti(jj,ii,rr)    Intermediate input tax
        tf(ff,ii,rr)    Factor tax
        tx(ii,ss,rr)    Export tax rate (defined on a net basis)
        tm(ii,ss,rr)    Import tariff rate
        tg(ii,rr)       Tax rates on government demand
        tp(ii,rr)       Tax rate on private demand

        vafm(jj,ii,rr)  Aggregate intermediate inputs
        vfm(ff,ii,rr)   Value of factor inputs (net of tax)
        vxmd(ii,rr,ss)  Value of commodity trade (fob - net export tax)
        vtwr(ii,rr,ss)  Transport services
        vst(ii,rr)      Value of international transport sales
        vdgm(ii,rr)     Government demand (domestic)
        vigm(ii,rr)     Government demand (imported)
        vdpm(ii,rr)     Aggregate private demands (domestic)
        vipm(ii,rr)     Aggregate private demands (domestic)

	vsave(rr)	Regional savings
	vkb(rr)		Value of capital stock (beginning of period)
	vdep(rr)	Value of capital depreciation

        ty0(i,r)        Output tax
        ti0(j,i,r)      Intermediate input tax
        tf0(f,i,r)      Factor tax
        ti0(i,j,r)      Intermediate input tax
        tx0(i,s,r)      Export tax rate (defined on a net basis)
        tm0(i,s,r)      Import tariff rate
        tg0(i,r)        Tax rates on government demand
        tp0(i,r)        Tax rate on private demand

        vafm0(j,i,r)    Aggregate intermediate inputs
        vfm0(f,i,r)     Value of factor inputs (net of tax)
        vxmd0(i,r,s)    Value of commodity trade (fob - net export tax)
        vtwr0(i,r,s)    Transport services
        vst0(i,r)       Value of international transport sales
        vdgm0(i,r)      Government demand (domestic)
        vigm0(i,r)      Government demand (imported)
        vdpm0(i,r)      Aggregate private demands (domestic)
        vipm0(i,r)      Aggregate private demands (domestic)

	vsave0(r)	Regional savings
	vkb0(r)		Value of capital stock (beginning of period)
	vdep0(r)	Value of capital depreciation;


$log Reading GTAP Data...

$offlisting
$batinclude ..\inclib\unzip %source% gms
$onlisting

*       Aggregate the value arrays:

$log Aggregating GTAP Data...

$batinclude ..\inclib\aggr vafm j i r vafm0
$batinclude ..\inclib\aggr vfm  f i r vfm0
$batinclude ..\inclib\aggr vxmd i r s vxmd0
$batinclude ..\inclib\aggr vtwr i r s vtwr0
$batinclude ..\inclib\aggr vst  i r   vst0
$batinclude ..\inclib\aggr vdgm i r   vdgm0
$batinclude ..\inclib\aggr vigm i r   vigm0
$batinclude ..\inclib\aggr vdpm i r   vdpm0
$batinclude ..\inclib\aggr vipm i r   vipm0

$batinclude ..\inclib\aggr vsave r   vsave0
$batinclude ..\inclib\aggr vkb   r   vkb0
$batinclude ..\inclib\aggr vdep  r   vdep0

*       Convert tax rates into values for aggregation:

parameter
        vxm(ii,rr)      Value of exports
        vdm(ii,rr)      Value of domestic production
        etax(ff,jj,ss)  Value of taxes on factor inputs to production
        mtax(ii,rr,ss)  Value of import tariffs
        xtax(ii,rr,ss)  Value of export taxes
        ytax(ii,rr)     Value of taxes on production outputs
        vtg(ii,rr)      Tax on government demand
        vti(jj,ii,rr)   Intermediate input tax
        vtp(ii,rr)      Taxes on private demand

        vxm0(i,r)       Value of exports
        vdm0(i,r)       Value of domestic production
        vgm0(i,r)       Value of public consumption
        vpm0(i,r)       Value of private consumption
        etax0(f,j,s)    Value of taxes on factor inputs to production
        mtax0(i,r,s)    Value of import tariffs
        xtax0(i,r,s)    Value of export taxes
        ytax0(i,r)      Value of taxes on production outputs
        vtg0(i,r)       Tax on government demand
        vti0(j,i,r)     Intermediate input tax
        vtp0(i,r)       Taxes on private demand;


vxm(ii,rr) = sum(ss, vxmd(ii,rr,ss));
vdm(ii,rr) = max(0, (sum(jj, vafm(jj,ii,rr)*(1+ti(jj,ii,rr))) 
                   + sum(ff, vfm(ff,ii,rr)*(1+tf(ff,ii,rr))))/(1-ty(ii,rr)) - vxm(ii,rr));

xtax(ii,rr,ss)$tx(ii,rr,ss) = tx(ii,rr,ss)*vxmd(ii,rr,ss);
mtax(ii,ss,rr)$tm(ii,ss,rr) = tm(ii,ss,rr)*(vxmd(ii,ss,rr)+xtax(ii,ss,rr)+vtwr(ii,ss,rr));
etax(ff,ii,rr)$tf(ff,ii,rr) = tf(ff,ii,rr)*vfm(ff,ii,rr);
vtp(ii,rr)$tp(ii,rr) = tp(ii,rr)*(vipm(ii,rr)+vdpm(ii,rr));
vtg(ii,rr)$tg(ii,rr) = tg(ii,rr)*(vigm(ii,rr)+vdgm(ii,rr));
vti(jj,ii,rr)$ti(jj,ii,rr) = ti(jj,ii,rr)*vafm(jj,ii,rr);
ytax(ii,rr)$ty(ii,rr) = ty(ii,rr) * (vxm(ii,rr)+vdm(ii,rr));

$batinclude ..\inclib\aggr xtax i r s  xtax0
$batinclude ..\inclib\aggr mtax i s r  mtax0
$batinclude ..\inclib\aggr etax f i r  etax0
$batinclude ..\inclib\aggr vtp  i r    vtp0
$batinclude ..\inclib\aggr vdm  i r    vdm0
$batinclude ..\inclib\aggr vtg  i r    vtg0
$batinclude ..\inclib\aggr vti  j i r  vti0
$batinclude ..\inclib\aggr ytax i r    ytax0

*       Reconvert to rates:

vxm0(i,r) = sum(s, vxmd0(i,r,s));
vgm0(i,r) = vigm0(i,r) + vdgm0(i,r);
vpm0(i,r) = vipm0(i,r) + vdpm0(i,r);

ty0(i,r)$(vdm0(i,r)+vxm0(i,r)) = ytax0(i,r) / (vdm0(i,r)+vxm0(i,r));
tx0(i,r,s)$vxmd0(i,r,s) = xtax0(i,r,s) / vxmd0(i,r,s);
tm0(i,s,r)$(vxmd0(i,s,r)+xtax0(i,s,r)+vtwr0(i,s,r))
        = mtax0(i,s,r)/(vxmd0(i,s,r)+xtax0(i,s,r)+vtwr0(i,s,r));
tf0(f,i,r)$vfm0(f,i,r) = etax0(f,i,r) / vfm0(f,i,r);
tg0(i,r)$vgm0(i,r)     = vtg0(i,r) / vgm0(i,r);
tp0(i,r)$vpm0(i,r)     = vtp0(i,r) / vpm0(i,r);
ti0(i,j,r)$vafm0(i,j,r) = vti0(i,j,r) / vafm0(i,j,r);

*	At this point, aggregation of the GTAP value arrays
*	has been completed.  Additional steps are performed
*	when source datasets include energy statistics:

$if defined eind $goto energy

*------------------------------------------------------------------------------
*       If the source data is purely economic statistics -- store arrays 
*	which have already been constructed and then complete the final report:

$batinclude ..\inclib\zip ..\data\ %dataset% ty0 -n ty ti0 -n ti tf0 -n tf tx0 -n tx tm0 -n tm tg0 -n tg tp0 -n tp vafm0 -n vafm vfm0 -n vfm vxmd0 -n vxmd vtwr0 -n vtwr vst0 -n vst vdgm0 -n vdgm vigm0 -n vigm  vdpm0 -n vdpm vipm0 -n vipm vdep0 -n vdep vsave0 -n vsave vkb0 -n vkb

$goto end

*------------------------------------------------------------------------------
$label energy
$log Aggregating Energy Data...

*       Aggregate and output energy quantities:

parameter
        EIND0(I,I,R)    Industrial energy demand (EJ&TKWH),
        EFD0(I,R)       Final energy demand (EJ&TKWH),
        EEXP0(I,R)      Energy exports (EJ&TKWH),
        EIMP0(I,R)      Energy imports (EJ&TKWH);

$batinclude ..\inclib\aggr eind i j r   eind0
$batinclude ..\inclib\aggr efd i r      efd0
$batinclude ..\inclib\aggr eexp i r     eexp0
$batinclude ..\inclib\aggr eimp i r     eimp0

*------------------------------------------------------------------------------
*	We only aggregate the IEO statistics from the gtapdoe dataset:

$if %source%==aspen5 $goto gtapdoe

$batinclude ..\inclib\zip ..\data\ %dataset% ty0 -n ty ti0 -n ti tf0 -n tf tx0 -n tx tm0 -n tm tg0 -n tg tp0 -n tp vafm0 -n vafm vfm0 -n vfm vxmd0 -n vxmd vtwr0 -n vtwr vst0 -n vst vdgm0 -n vdgm vigm0 -n vigm  vdpm0 -n vdpm vipm0 -n vipm vdep0 -n vdep vsave0 -n vsave vkb0 -n vkb eind0 -n eind efd0 -n efd eexp0 -n eexp eimp0 -n eimp 
$goto end

$label gtapdoe

*	Check that the target dataset has only two primary factors: labor and capital

set	fchk(*) /lab, cap/;
abort$(card(f) ne 2) "*** Error: set f must be /lab,cap/";
fchk(f) = no;
abort$card(fchk) "*** Error: to aggregate to an energy-based dataset, f must be /lab,cap/";

$log Aggregating IEO projections...

*	Aggregation of GDP and carbon emissions profiles based on 
*	data from the the DOE International Energy Outlook:

SET	tt	Time periods include 1990
		/ 1990,1995,2000,2005,2010,2015,2020,2025,2030/;

SET	t(tt)	Time period including the base year
  		/ 1995,2000,2005,2010,2015,2020,2025,2030/;


SET	sc	Energy scenarios in the IEO statistics /ref, lo, hi/;

parameter       cecphys(*)      carbon emission coefficients for physical units 
                                /col 0.024, gas 0.0137, oil 0.0181 /;

parameter
	IEOCARB		IEO carbon emissions
	IEOCRUD		IEO crude production
	IEOPRIC		IEO energy prices
	ED(ii,rr)	Base year energy demand
	ES(ii,rr)	Base year energy supply

	IEOENERGY	Implied IEO energy profile by region;


*	-----------------------------------------------------------
*	Read the IEO statistics to produce projections for energy production and demand:

$include ieo.dat

*	Energy demand and supply in GTAP_DOE:

ed(ii,rr) = efd(ii,rr) + sum(jj, eind(ii,jj,rr));
es(ii,rr) = eexp(ii,rr) + ed(ii,rr) - eimp(ii,rr);

*	Finally, generate consumption and production indices:


set ffuel(ii) /oil, gas, col/;

*	Generate an index of baseline carbon emissions:

ieocarb(sc,ffuel,r,tt)$sum(rr$mapr(rr,r), ed(ffuel,rr))
		= sum(rr$mapr(rr,r), ed(ffuel,rr)*ieocarbon(sc,ffuel,rr,tt))
		/ sum(rr$mapr(rr,r), ed(ffuel,rr));

*	Generate a paralell index of energy demand:

ieoenergy(sc,rr,tt) = sum(ffuel, ed(ffuel,rr) * ieocarbon(sc,ffuel,rr,tt))
		    / sum(ffuel, ed(ffuel,rr));

*	Generate a crude oil production index:

loop(r$sum(rr$mapr(rr,r), es("cru",rr)),
  ieocrud(sc,r,t)	= sum(rr$mapr(rr,r), es("cru",rr)*ieocrude(sc,rr,t)) 
		/ sum(rr$mapr(rr,r), es("cru",rr));
);

display es, ieocrude, ieocrud;

*	Input growth rates and calibrate AEEI to reconcile GDP growth rates with energy
*	and carbon emissions projections.

Parameter
	gdp		gdp index in source gtapdoe dataset
	gdp_adj(sc)	GDP adjustment multipliers /ref 1, lo 1, hi 1/;

* Note that it's changed for GTAP5

Table GROW(*,*) Time path of GDP growth rates

      1995  2000  2005  2010  2015  2020  2025  2030

AUS   2.3   2.3   2.2   2.1   2.0   1.9   1.8   1.7   
NZL   2.3   2.3   2.2   2.1   2.0   1.9   1.8   1.7   
CHN   6.0   6.0   5.5   5.0   5.0   4.5   4.5   4.0
HKG   6.0   6.0   5.5   5.0   5.0   4.5   4.5   4.0
JPN   2.1   2.1   2.2   2.1   1.9   1.9   1.8   1.7
KOR   3.0   3.0   5.5   5.0   5.0   4.5   4.5   4.5 
TWN   6.0   6.0   5.5   5.0   5.0   4.5   4.5   4.0 
IDN   3.0   3.0   5.5   5.0   5.0   4.5   4.5   4.5
MYS   3.0   3.0   5.5   5.0   5.0   4.5   4.5   4.5
PHL   3.0   3.0   5.5   5.0   5.0   4.5   4.5   4.5
SGP   3.0   3.0   5.5   5.0   5.0   4.5   4.5   4.5
THA   3.0   3.0   5.5   5.0   5.0   4.5   4.5   4.5
VNM   3.0   3.0   5.5   5.0   5.0   4.5   4.5   4.5
BGD   5.0   5.0   5.0   5.0   5.0   4.5   4.5   4.5
IND   5.0   5.0   5.0   5.0   5.0   4.5   4.5   4.5
LKA   5.0   5.0   5.0   5.0   5.0   4.5   4.5   4.5
XSA   5.0   5.0   5.0   5.0   5.0   4.5   4.5   4.5
CAN   2.3   2.3   2.2   2.2   2.1   1.95  1.8   1.7
USA   2.7   2.7   2.4   2.2   2.0   1.9   1.8   1.7
MEX   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
XCM   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
COL   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
PER   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
VEN   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
XAP   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
ARG   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
BRA   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
CHL   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
URY   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
XSM   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
AUT   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
DNK   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
FIN   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
FRA   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
DEU   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
GBR   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
GRC   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
IRL   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
ITA   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
NLD   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
PRT   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
ESP   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
SWE   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
BEL   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
LUX   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
CHE   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
XEF   2.2   2.2   2.2   2.1   2.0   1.9   1.8   1.7
HUN  -1.5   3.3   3.3   3.3   3.0   2.7   2.5   2.3 
POL  -1.5   3.3   3.3   3.3   3.0   2.7   2.5   2.3 
XCE  -1.5   3.3   3.3   3.3   3.0   2.7   2.5   2.3 
XSU  -3.6   3.6   3.6   3.6   3.6   3.3   3.0   2.7 
TUR   2.3   2.3   2.2   2.1   2.0   1.9   1.8   1.7
XME   3.2   3.2   3.2   3.2   3.2   3.2   3.2   3.2
MAR   3.2   3.2   3.2   3.2   3.2   3.2   3.2   3.2
XNF   3.2   3.2   3.2   3.2   3.2   3.2   3.2   3.2
BWA   2.3   2.3   2.2   2.1   2.0   1.9   1.8   1.7
XSC   2.3   2.3   2.2   2.1   2.0   1.9   1.8   1.7
MWI   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
MOZ   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
TZA   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
ZMB   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
ZWE   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
XSF   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
UGA   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
XSS   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0
XRW   4.0   4.0   4.0   4.0   4.0   3.5   3.5   3.0;  


gdp(sc,t,rr) = 1 ;
Display gdp ;
loop(t, gdp(sc,t+1,rr) = gdp(sc,t,rr) * (1+gdp_adj(sc)*grow(rr,t)/100)**5;);
Display gdp ;
gdp(sc,t,rr)$gdp(sc,"1995",rr) = gdp(sc,t,rr) / gdp(sc,"1995",rr);
Display gdp ;


parameter	egrow	energy growth rates in the ieo dataset;
egrow(sc,rr,t+1)$ieoenergy(sc,rr,t) = round( 100 * ((ieoenergy(sc,rr,t+1)/ieoenergy(sc,rr,t))**(1/5) - 1), 1);
option egrow:1:1:1;
display egrow;

*	Now in each scenario we attribute differences in emissions
*	profiles to differences in GDP growth rate and holding average
*	AEEI equal to the assumed value:

*	(NB There will still be differences in the fuel-specific AEEI values,
*	reflecting assumptions about fuel-switching through the horizon.)

parameter	
		aeei	Fuel-specific energy efficiency index in GTAPDOE dataset,
		grow	gdp growth rates in source gtapdoe dataset,
		fgdp	factor-based regional gdp in gtapdoe dataset
		ieogdp	gdp index in aggregate dataset,
		aeei0	energy efficiency index in aggregated dataset,
		grow0	implied gdp growth rates in aggregated data;

aeei(sc,ffuel,t,rr) = ieocarbon(sc,ffuel,rr,t) / gdp(sc,t,rr);

option aeei:2:2:2;
display aeei;


fgdp(rr) = sum((ff,ii), vfm(ff,ii,rr));
ieogdp(sc,t,r) = sum(rr$mapr(rr,r), gdp(sc,t,rr)* fgdp(rr))
	    /  sum(rr$mapr(rr,r), fgdp(rr));

display ed;

aeei0(sc,ffuel,t,r)$sum(rr$mapr(rr,r), ed(ffuel,rr))
	 = sum(rr$mapr(rr,r), aeei(sc,ffuel,t,rr)*ed(ffuel,rr))
		  /  sum(rr$mapr(rr,r), ed(ffuel,rr));


grow0(sc,r,t+1) = round( 100 * ((ieogdp(sc,t+1,r)/ieogdp(sc,t,r))**(1/5) - 1), 1);
option grow0:1:1:1;
display grow0;

*	EFT is made up of Norway (101), Switzerland (92), and Iceland (110) -- PMB

parameter kyoto Kyoto targets /
AUS 108
NZL 100
JPN 94
CAN 94
USA 93
AUT 92 
DNK 92  
FIN 92  
FRA 92  
DEU 92  
GBR 92  
GRC 92  
IRL 92  
ITA 92  
NLD 92  
PRT 92  
ESP 92  
SWE 92  
BEL 92
LUX 92
CHE 92  
XEF 103  
HUN 94  
POL 94  
XCE 94  
XSU 100 /;



parameter	kyoto0	Kyoto targets for aggregate regions;

kyoto0(r) = sum(rr$mapr(rr,r), sum(ffuel, ed(ffuel,rr)* cecphys(ffuel))*kyoto(rr))
	/   sum(rr$mapr(rr,r), sum(ffuel, ed(ffuel,rr)* cecphys(ffuel)));

display kyoto0;

*	Assign sets which define which regions are Annex B and which are OECD

sets	annexb(rr)   Annex B regions /
 AUS    Australia
 NZL    New Zealand
 JPN    Japan
 CAN    Canada
 USA    United States of America
AUT Austria                              
DNK Denmark                              
FIN Finland                              
FRA France                               
DEU Germany                              
GBR United Kingdom                       
GRC Greece                               
IRL Ireland                              
ITA Italy                                
NLD Netherlands                          
PRT Portugal                             
ESP Spain                                
SWE Sweden                               
BEL Belgium 
LUX Luxembourg               
CHE Switzerland                          
XEF rest of EFTA                         
HUN Hungary                              
POL Poland                               
XCE rest of Central European associates  
XSU former Soviet Union /;

set	oecd(rr)     OECD member regions Korea not included /
 AUS    Australia
 NZL    New Zealand
 JPN    Japan
 CAN    Canada
 USA    United States of America
AUT Austria                              
DNK Denmark                              
FIN Finland                              
FRA France                               
DEU Germany                              
GBR United Kingdom                       
GRC Greece                               
IRL Ireland                              
ITA Italy                                
NLD Netherlands                          
PRT Portugal                             
ESP Spain                                
SWE Sweden                               
BEL Belgium 
LUX Luxembourg               
CHE Switzerland                          
XEF rest of EFTA /;

set	EU(rr)	EU countries /
AUT Austria                              
DNK Denmark                              
FIN Finland                              
FRA France                               
DEU Germany                              
GBR United Kingdom                       
GRC Greece                               
IRL Ireland                              
ITA Italy                                
NLD Netherlands                          
PRT Portugal                             
ESP Spain                                
SWE Sweden                               
BEL Belgium 
LUX Luxembourg               
CHE Switzerland                          
XEF rest of EFTA /;

set annexb0, oecd0;
Set
	EU0	EU countries and regions
	EIT	Economies in transition
	NEUAB	Non-EU Annex B
	EUFSU	EU and FSU
;

*	Use GDP as determinant:

annexb0(r) = yes$(sum(mapr(rr,r)$annexb(rr), fgdp(rr)      ) gt 
		  sum(mapr(rr,r)$(not annexb(rr)), fgdp(rr))     );

oecd0(r) = yes$(sum(mapr(rr,r)$oecd(rr)      , fgdp(rr)) gt 
                sum(mapr(rr,r)$(not oecd(rr)), fgdp(rr)));

EU0(r) = yes$(sum(mapr(rr,r)$EU(rr)      , fgdp(rr)) gt 
                sum(mapr(rr,r)$(not EU(rr)), fgdp(rr)));
if (card(eu0) eq 0, eu0("phantom") = yes;);

EIT(r) = yes$(annexb0(r) AND not oecd0(r)) ;
NEUAB(r) = yes$(annexb0(r) AND not EU0(r)) ;
EUFSU(r) = yes$(EIT(r) OR EU0(r)) ;

display annexb0, oecd0, EU0, EIT, NEUAB, EUFSU;

$batinclude ..\inclib\zip ..\data\ %dataset% ty0 -n ty ti0 -n ti tf0 -n tf tx0 -n tx tm0 -n tm tg0 -n tg tp0 -n tp vafm0 -n vafm vfm0 -n vfm vxmd0 -n vxmd vtwr0 -n vtwr vst0 -n vst vdgm0 -n vdgm vigm0 -n vigm  vdpm0 -n vdpm vipm0 -n vipm eind0 -n eind efd0 -n efd eexp0 -n eexp eimp0 -n eimp ieoprice ieocrud -n ieocrude ieocarb -n ieocarbon ieogdp annexb0 -n annexb oecd0 -n oecd EU0 -n EU kyoto0 -n kyoto

$label end

*------------------------------------------------------------------------------
*       Generate a formatted GDP report:

parameter
        pmx0(i,s,r)     Reference price of imports,
        pmt0(i,s,r)     Reference price of transport services,
        pg0(i,r)        Reference price of government demand,
        pc0(i,r)        Reference price for private demand,
        py0(i,r)        Reference output price (domestic or export),
        pai0(j,i,r)     Reference price for intermediate inputs,
        pf0(f,i,r)      Reference price for factor inputs;

pg0(i,r) = 1 + tg0(i,r);
pc0(i,r) = 1 + tp0(i,r);
pmt0(i,s,r) = (1 + tm0(i,s,r));
pmx0(i,s,r) = (1 + tm0(i,s,r)) * (1 + tx0(i,s,r));
py0(i,r) =1 - ty0(i,r);
pf0(f,j,s) = 1 + tf0(f,j,s);

parameter       sector          Economic activity by sector,
                region          Economic activity by region,
                order           row order for sorting;

sector(i,"gdp") = sum((f,r), pf0(f,i,r)*vfm0(f,i,r));
sector(i,"gdp%") = 100 * sector(i,"gdp") / sum(j, sector(j,"gdp"));

region(r,"gdp") = sum((i,f), pf0(f,i,r)*vfm0(f,i,r));
region(r,"gdp%") = 100 * region(r,"gdp") / sum(s, region(s,"gdp"));

sector(i,"trade") = sum((r,s), pmx0(i,r,s)*vxmd0(i,r,s)+pmt0(i,r,s)*vtwr0(i,r,s));
sector(i,"trade%") = 100 * sector(i,"trade") / sum(j, sector(j,"trade"));

region(r,"trade") = sum((i,s), pmx0(i,r,s)*vxmd0(i,r,s)+pmt0(i,r,s)*vtwr0(i,r,s));
region(r,"trade%") = 100 * region(r,"trade") / sum(s, region(s,"trade"));

display sector, region;

*       Generate a formatted report as well:

$set filename %dataset%
file gdprep /%filename%.ech/; put gdprep //; gdprep.nd=1;

$setglobal row_order order

order(i) = sector(i,"trade");
$if     exist ..\inclib\gams2tbl $batinclude ..\inclib\gams2tbl sector
$if not exist ..\inclib\gams2tbl $libinclude gams2tbl sector
order(i) = 0;

order(r) = region(r,"trade");
$if     exist ..\inclib\gams2tbl $batinclude ..\inclib\gams2tbl region
$if not exist ..\inclib\gams2tbl $libinclude gams2tbl region
order(r) = 0;
putclose;

*       Put out some summary statistics on the screen:

file kcon /con/; put kcon //; kcon.nd = 1;

order(i) = sector(i,"trade");
$if     exist ..\inclib\gams2tbl $batinclude ..\inclib\gams2tbl sector
$if not exist ..\inclib\gams2tbl $libinclude gams2tbl sector
order(i) = 0;

order(r) = region(r,"trade");
$if     exist ..\inclib\gams2tbl $batinclude ..\inclib\gams2tbl region
$if not exist ..\inclib\gams2tbl $libinclude gams2tbl region
order(r) = 0;

put //"Summary tables written to %filename%.ech."/;
