set rb(r)       dynamic set used to define the current region;

positive
variables       investz         calibrated invest
                vpmz(i)         calibrated vpm
                vgmz(i)         calibrated vgm
                vdmz(i)         calibrated vdm
                vafmz(j,i)      calibrated vadm
                vfmz(f,i)       calibrated vfm
                m0z(d,i)        calibrated m0
                d0z(d,i)        calibrated d0;

variable        obj;

set seti(d) /i/, setg(d)/g/, setc(d)/c/;

equations       profit          benchmark consistency - zero profit
                dommkt          benchmark consistency - domestic market
                imports         benchmark consistency - import supply
                interm          benchmark consistency - intermediate demand
                consum          benchmark consistency - consumer imports
                govern          benchmark consistency - government imports

                objbal          defines the objective function;

objbal..        obj =e= 10 * sum(rb, sqr(investz-vdm("cgd",rb)) +
                        sum(i,   sqr(vpmz(i)-vpm(i,rb))
                        +        sqr(vgmz(i)-vgm(i,rb))
                        +        sqr(vdmz(i)-vdm(i,rb)) 
                        + sum(j, sqr(vafmz(j,i)-vafm(j,i,rb)))
                        + sum(f, sqr(vfmz(f,i)-vfm(f,i,rb)))
                        + sum(d, sqr(m0z(d,i)-m0(d,i,rb))
                        +        sqr(d0z(d,i)-d0(d,i,rb)))));

profit(i,rb)..  (vdmz(i)+vxm(i,rb))*(1-ty(i,rb)) =e= sum(j,vafmz(j,i)*(1+ti(j,i,rb)))  
                                                   + sum(f, vfmz(f,i)*(1+tf(f,i,rb)));
dommkt(i,rb)..  vdmz(i) =e= sum(d, d0z(d,i)) + investz$cgd(i);
imports(i,rb).. sum(d, m0z(d,i)) =e= vim(i,rb);
interm(i,rb)..  d0z("i",i) + m0z("i",i) =e= sum(j, vafmz(i,j));
consum(i,rb)..  d0z("c",i) + m0z("c",i) =e= vpmz(i);
govern(i,rb)..  d0z("g",i) + m0z("g",i) =e= vgmz(i);

*	Declare the least-squares calibration problem as a nonlinear program
*	with the objective to minimize OBJ:

model regbal /profit,dommkt,imports,interm,consum,govern,objbal/;

variables       pzprofit                benchmark consistency - zero profit
                pzdommkt                benchmark consistency - domestic market
                pzimport                benchmark consistency - import supply
                pzinterm                benchmark consistency - intermediate demand
                pzconsum                benchmark consistency - consumer imports
                pzgovern                benchmark consistency - government imports

equations ezvdm, ezvafm, ezvfm, ezm0, ezd0, ezvpm, ezvgm, ezinvest;

ezvdm(i)..      sum(rb, -pzprofit(i,rb)*(1-ty(i,rb)) - pzdommkt(i,rb)) 
			=g= -sum(rb, 20 * (vdmz(i) - vdm(i,rb)));

ezvafm(j,i)..   sum(rb, pzprofit(i,rb) * (1 + ti(j,i,rb)) + pzinterm(j,rb)) 
			=g= -sum(rb, 20 * (vafmz(j,i)-vafm(j,i,rb)));

ezvfm(f,i)..    sum(rb, pzprofit(i,rb) * (1 + tf(f,i,rb))) 
			=g= -20 * sum(rb, vfmz(f,i) - vfm(f,i,rb));

ezm0(d,i)..     sum(rb, -pzimport(i,rb) - pzinterm(i,rb)$SETi(d) 
			- pzconsum(i,rb)$SETc(d) - pzgovern(i,rb)$SETg(d))
			=g= -sum(rb, 20 * (m0z(d,i) - m0(d,i,rb)));

ezd0(d,i)..     sum(rb, +pzdommkt(i,rb) - pzinterm(i,rb)$SETi(d) 
			- pzconsum(i,rb)$SETc(d) - pzgovern(i,rb)$SETg(d))
			=g= -sum(rb, 20 * (d0z(d,i) - d0(d,i,rb)));

ezvpm(i)..      sum(rb, 20 * (vpmz(i)-vpm(i,rb)) + pzconsum(i,rb)) =e= 0;

ezvgm(i)..      sum(rb, 20 * (vgmz(i)-vgm(i,rb)) + pzgovern(i,rb)) =e= 0;

ezinvest..      sum(rb, 20 * (investz-vdm("cgd",rb)) + sum(cgd, pzdommkt(cgd,rb))) =e= 0;

*	Declare the regional balance problem as an MCP:

model regbalmcp/ 
        profit.pzprofit, dommkt.pzdommkt, imports.pzimport, 
        interm.pzinterm, consum.pzconsum, govern.pzgovern,
        ezvdm.vdmz, ezvafm.vafmz, ezvfm.vfmz, ezm0.m0z, 
        ezd0.d0z, ezvpm.vpmz, ezvgm.vgmz, ezinvest.investz/;
