program compile

use mpsge

	implicit none

	character (len=255) :: fname

!!	Read file name and model name from the command line:

!        call getcl(fname)
        open(iogms,file='basic.mps')

!      Master routine to read an MPSGE input file:
!        - Read one line at a time into a linked list for listing.
!        - Locate functions and set up pointers to the constituent records.
!        - Parse records into fields.
!
	call readmps

!      Parse all fields in the file into constituent elements:
!      - Field identifier text
!      - Identifier argument text
!      - Spanning domain text
!      - Condition text.

 	call parse

!	Add sector ids to the symbol table and verify that
!	multiple prod blocks have not be included.

	call sectors

!	Check input syntax on prod: records:

	call chkprod

!	Read the nesting assignments from the $prod line:

	call readnest

!	Read production function coefficients and set up list of
!	nest elements:

	call readcoef

!	Write out declarations for various values:

	call declarations	

!	Generate an echoprint of the input file and the functions:

	call echoprint
	call fnecho


contains

	subroutine fnecho
	type (field), pointer :: f, fnf
	type (record), pointer :: r
	type (function), pointer :: fn
	type (nest), pointer :: n

	fn => functionlist
	do while (associated(fn))
	  r => fn%firstrecord
	  fnf => r%firstfield
	  if (fnf%label.ne.'$prod') then
	    fn => fn%next
	    cycle
	  end if
	  write(*,'(a)') ' Production function: '//trim(fnf%data)
	  write(*,'(a,i3)') ' nrecord = ',fn%nrecord
	  write(*,'(a,i3)') ' nfield = ',fn%nfield
	  write(*,'(a)') ' Nests: '
	  n => fn%inputs
	  do while (associated(n))
	    write(*,'(a,i3)') ' '//trim(n%ident)//' Level: ', n%level
	    if (associated(n%subnests)) then
	      call nlist(n%subnests)
	    end if
	    if (associated(n%elements)) then
	      call elist(n%elements)
	    end if
	    n => n%next
	  end do
	  fn => fn%next
	end do
	return
	end subroutine fnecho

	subroutine nlist(sn)
	type (nestlist), pointer :: sn, s
	s => sn
	do while (associated(s))
	  write(*,'(t15,a)') trim(s%n%ident)
	  s => s%next
	end do
	return
	end subroutine nlist

	subroutine elist(el)
	type (netputlist), pointer :: el, e
	character (len=255) :: domain, subdomain
	e => el
	do while (associated(e))
	  call writedomain(e%np%d, domain)
	  call writedomain(e%np%sd, subdomain)
	  write(*,'(t15,a)') trim(e%np%f%data)//' ('//trim(domain)//') ('//trim(subdomain)//')'
	  e => e%next
	end do
	return
	end subroutine elist

	subroutine declarations
	type (field), pointer :: f, fnf
	type (record), pointer :: r
	type (function), pointer :: fn
	type (nest), pointer :: n
	type (netputlist), pointer :: e
	integer :: fnindex = 0
	character (len=1024) :: rec

	write(*,'(//a/)') '*       Benchmark Costs:'
	fn => functionlist
	do while (associated(fn))
	  r => fn%firstrecord
	  fnf => r%firstfield
	  if (fnf%label.ne.'$prod') then
	    fn => fn%next
	    cycle
	  end if
	  if (fn%index.eq.0) then
	    fn%index = fnindex + 1
	    fnindex = fnindex + 1
	  end if
	  call writeparam('c0_',fn%index,fnf%arg,rec)
	  write(*,'(a)') 'PARAMETER '//trim(rec)//'  "Benchmark cost for sector '//trim(fnf%data)//'";'
	  rec = trim(rec)//' = '
	  n => fn%inputs
	  do while (associated(n))
	    e => n%elements
	    do while (associated(e))
	      call vsum(e%np%sd, e%np%qref, e%np%pref, e%f, rec)
	      e => e%next
	    end do
	    n => n%next
	  end do
	  write(*,'(a)') trim(rec)//';'
	  fn = fn%next
	end do

	return
	end subroutine declarations

	subroutine vsum(d, qref_, pref_, condtn_, string)
	character (len=*) :: string
	type (domain), pointer :: d
	type (field), pointer :: qref_, pref_, condtn_
	character (len=255) :: domstr, qref, pref, condtn

	condtn = ' '
	call formv(qref_,'1',qref)
	call formv(pref_,'1',pref)

!	if (associated(condtn_)) condtn = condtn_%condtn
	if (condtn.ne.' ')       condtn = '$'//trim(condtn)

	if (.not.associated(d)) then
	  string = trim(string)//'+('//trim(qref)//'*'//trim(pref)//')'//trim(condtn)
	  return
	end if
	call writedomain(d, domstr)
	if (dimension(d).gt.1) domstr = '('//trim(domstr)//')'
	string = trim(string)//'+sum('//trim(domstr)//trim(condtn)//', '//trim(qref)//'*'//trim(pref)//')'
	return
	end subroutine vsum

	subroutine formv(f,default,value)
	type (field), pointer :: f
	character (len=*), intent(in) :: default
	character (len=*), intent(out) ::  value
	if (associated(f)) then
	  value = '('//f%data//')'
	  if (f%condtn.ne.' ') &
	    value = '('//trim(value)//'$'//trim(f%condtn)//' + '//trim(default)//'$( not('//trim(f%condtn)//')))'
	else
	  value = trim(default)
	endif
	return
	end subroutine formv

	end program compile
