$title Joint model

* ----------------------------------------------------------------------------
* Closed economy with joint production.
* Laurent Cretegny, CoPS, Monash University, Australia, 2004.
* ----------------------------------------------------------------------------

$if not exist joint.gdx $call gdxxrw i=joint.xls o=joint.gdx index=declare

sets
  s                 sector
  o                 commodity
  f                 primary factor;

parameters
  sam(*,*)          benchmark social accounting matrix
  supply(s,o)       supply of output [[sppl:s*o]]
  interm(o,s)       intermediate demand [[intr:o*s]]
  factor(f,s)       factor demand [[fctr:f*s]]
  demand(o)         demand for consumption [[dmnd:o]]
  govdem(o)         government demand [[gvdm:o]]
  endow(f)          endowment [[endw:f]]
  etrn(s)           elasticity of transformation [[etrn:s]]
  esub(s)           elasticity of substitution [[esub:s]]
  esubc             elasticity of substitution for consumption [[esbc:]];

$gdxin joint
$load s o f etrn esub esubc sam
$gdxin

supply(s,o) = sam(s,o);
interm(o,s) = sam(o,s);
factor(f,s) = sam(f,s);
demand(o) = sam(o,"ra");
endow(f) = sam("ra",f);

$if not exist joint.har $include gdxtohar

$ontext
$model:joint

$sectors:
  y(s)              ! production
  u                 ! utility index

$commodities:
  pd(o)             ! domestic price of commodity
  pf(f)             ! price of primary factor
  pu                ! price index for utility

$consumers:
  ra                ! representative agent income

$prod:y(s)          t:etrn(s)           va:esub(s)
  o:pd(o)           q:supply(s,o)
  i:pd(o)           q:interm(o,s)
  i:pf(f)           q:factor(f,s)       va:

$prod:u             s:esubc
  o:pu              q:(sum(o, demand(o)))
  i:pd(o)           q:demand(o)

$demand:ra
  d:pu
  e:pf(f)           q:endow(f)

$report:
  v:rsupply(s,o)    o:pd(o)             prod:y(s)
  v:rdemand(o)      i:pd(o)             prod:u

$offtext
$sysinclude mpsgeset joint

* Benchmark replication

joint.iterlim = 0;
$include joint.gen
solve joint using mcp;
abort$(abs(joint.objval) gt 1e-7) "*** joint does not calibrate ! ***";
joint.iterlim = 1000;

* Fixation of a numeraire to allow comparison

pu.fx = pu.l;

* Counterfactual : 10% increase in labor endowment

endow("lab") = endow("lab") * 1.1;

$include joint.gen
solve joint using mcp;

* Results

parameters
  res_com           results by commodity in % change from basis
  res_agg           aggregate results in % change from basis; 

res_com("sagr",o) = (rsupply.l("sagr",o)/supply("sagr",o) - 1) * 100;
res_com("sind",o) = (rsupply.l("sind",o)/supply("sind",o) - 1) * 100;
res_com("demand",o) = (rdemand.l(o)/demand(o) - 1) * 100;
res_com("price",o) = (pd.l(o) - 1) * 100;

res_agg("u") = (u.l - 1) * 100;
res_agg("pu") = (pu.l - 1) * 100;
res_agg("ra_inc") = (ra.l/(sum(o, demand(o))) - 1) * 100;

option decimals = 6;
display res_com, res_agg;
