$title Joint model

$ontext
--------------------------------------------------------------------------------
Closed 2x2 economy with joint production.
Laurent Cretegny, CoPS, Monash University, Australia, 2003.
--------------------------------------------------------------------------------
$offtext

sets
  s                 sector                        / s_agr, s_ind /
  o                 commodity                     / agr, ind /
  f                 primary factor                / lab, cap /;

parameters
  sam(*,*)          benchmark social accounting matrix
  factor(f,s)       factor demand
  interm(o,s)       intermediate demand
  demand(o)         demand for consumption
  endow(f)          endowment
  tr_y(s,o)         tax revenue on output
  supply(s,o)       supply of output
  cons              aggregate consumption
  t_y(s,o)          tax rate on output
  t_y_l(s,o)        power of tax on output;

$call gdxxrw i=joint.xls o=joint.gdx index=declare
$gdxin joint.gdx
$load sam = sam
$gdxin

factor(f,s) = sam(f,s);
interm(o,s) = sam(o,s);
demand(o) = sam(o,"ra");
endow(f) = sam("ra",f);
tr_y("s_ind",o) = sam("ty",o);
supply(s,o) = sam(s,o) + tr_y(s,o);
cons = sum(o, demand(o));
t_y(s,o) = tr_y(s,o) / supply(s,o);
t_y_l(s,o) = 1 - t_y(s,o);

$ontext
$model:joint

$sectors:
  y(s)              ! production
  c                 ! aggregate consumption

$commodities:
  p(o)              ! price of commodity
  pc                ! price of aggregate consumption
  pf(f)             ! price of primary factor

$consumers:
  ra                ! representative agent income

$prod:y(s)          t:1.0               s:0.0           va:1.0
  o:p(o)            q:supply(s,o)       p:t_y_l(s,o)    a:ra          t:t_y(s,o)
  i:pf(f)           q:factor(f,s)       va:
  i:p(o)            q:interm(o,s)

$prod:c             s:1.0
  o:pc              q:cons
  i:p(o)            q:demand(o)

$demand:ra
  d:pc              q:cons
  e:pf(f)           q:endow(f)

$report:
  v:r_supply(s,o)   o:p(o)              prod:y(s)
  v:r_cons(o)       i:p(o)              prod:c

$offtext
$sysinclude mpsgeset joint

* Benchmark replication

joint.iterlim = 0;
$include joint.gen
solve joint using mcp;
abort$(abs(joint.objval) gt 1e-7) "*** joint does not calibrate ! ***";
joint.iterlim = 1000;

* Fixation of a numeraire to allow comparison

ra.fx = ra.l;

* Counterfactual : 10% increase in tax rate on industrial output

t_y(s,o) = t_y(s,o) * 1.1;

$include joint.gen
solve joint using mcp;

* Results

parameters
  res_com           results by commodity in % change from basis
  res_agg           aggregate results in % change from basis; 

res_com("s_agr",o) = (r_supply.l("s_agr",o)/supply("s_agr",o) - 1) * 100;
res_com("s_ind",o) = (r_supply.l("s_ind",o)/supply("s_ind",o) - 1) * 100;
res_com("cons",o) = (r_cons.l(o)/demand(o) - 1) * 100;
res_com("price",o) = (p.l(o) - 1) * 100;

res_agg("c") = (c.l - 1 ) * 100;
res_agg("pc") = (pc.l - 1 ) * 100;
res_agg("ra_inc") = (ra.l/cons - 1 ) * 100;

option decimals = 6;
display res_com, res_agg;
