$title Basic model

$ontext
--------------------------------------------------------------------------------
Closed 2x2 economy.
Laurent Cretegny, CoPS, Monash University, Australia, 2004.
--------------------------------------------------------------------------------
$offtext

$if not exist basic.gdx $call gdxxrw i=basic.xls o=basic.gdx index=declare

sets
  s                 commodity and sector
  f                 primary factor;

parameters
  sam(*,*)          benchmark social accounting matrix
  factor(f,s)       factor demand [[fac:f*s]]
  demand(s)         demand for consumption [[dem:s]]
  endow(f)          endowment [[end:f]]
  supply(s)         supply of output
  cons              aggregate consumption;

$gdxin basic
$load s f sam
$gdxin

factor(f,s) = sam(f,s);
demand(s) = sam(s,"ra");
endow(f) = sam("ra",f);
supply(s) = sum(f, factor(f,s));
cons = sum(s, demand(s));

$if not exist basic.har $include gdxtohar

$ontext
$model:basic

$sectors:
  y(s)              ! production
  c                 ! aggregate consumption

$commodities:
  p(s)              ! price of commodity
  pc                ! price of aggregate consumption
  pf(f)             ! price of primary factor

$consumers:
  ra                ! representative agent income

$prod:y(s)          s:1.0
  o:p(s)            q:supply(s)
  i:pf(f)           q:factor(f,s)

$prod:c             s:1.0
  o:pc              q:cons
  i:p(s)            q:demand(s)

$demand:ra
  d:pc              q:cons
  e:pf(f)           q:endow(f)

$report:
  v:r_cons(s)       i:p(s)              prod:c

$offtext
$sysinclude mpsgeset basic

* Benchmark replication

basic.iterlim = 0;
$include basic.gen
solve basic using mcp;
abort$(abs(basic.objval) gt 1e-8) "*** basic does not calibrate ! ***";
basic.iterlim = 1000;

* Fixation of a numeraire to allow comparison

ra.fx = ra.l;

* Counterfactual : 50% increase in labor endowment

endow("lab") = 1.5 * endow("lab");

$include basic.gen
solve basic using mcp;

* Results

parameters
  res_com           results by commodity in % change from basis
  res_agg           aggregate results in % change from basis; 

res_com("output",s) = (y.l(s) - 1) * 100;
res_com("cons",s) = (r_cons.l(s)/demand(s) - 1) * 100;
res_com("price",s) = (p.l(s) - 1) * 100;

res_agg("c") = (c.l - 1 ) * 100;
res_agg("pc") = (pc.l - 1 ) * 100;
res_agg("ra_inc") = (ra.l/cons - 1 ) * 100;

option decimals = 6;
display res_com, res_agg;
