$title Demo model

* ------------------------------------------------------------------------------
* Demonstration model characterizing MPSGE features.
* Laurent Cretegny, CoPS, Monash University, Australia, 2004.
* ------------------------------------------------------------------------------

$if not exist demo.gdx $call gdxxrw i=demo.xls o=demo.gdx index=declare

sets
  h                 household
  s                 sector
  o                 commodity
  f                 primary factor;

parameters
  sam(*,*)          benchmark social accounting matrix
  interm(o,s)       intermediate demand [[intr:o*s]]
  factor(f,s)       factor demand [[fctr:f*s]]
  totfac            total factor demand [[ttfc:]]
  demand(o,h)       demand for consumption [[dmnd:o*h]]
  totdem(h)         total consumption demand [[ttdm:h]]
  endow(f,h)        endowment [[endw:f*h]]
  govdem(o)         government demand [[gvdm:o]]
  supply(s,o)       supply of output [[sppl:s*o]]
  etrn(s)           elasticity of transformation [[etrn:s]]
  esub(s)           elasticity of substitution [[esub:s]]
  t_p(o)            tax rate on output [[tp:o]]
  t_pf(f,s)         tax rate on factor [[tpf:f*s]]
  t_p1(o,h)         tax rate on consumption [[tp1:o*h]]
  t_p2(o,h)         tax rate on consumption [[tp2:o*h]]
  tp_pf(f,s)        power of tax on factor
  tp_p(o,h)         power of tax on consumption;

$gdxin demo
$load h s o f etrn esub sam
$gdxin

interm(o,s) = sam(o,s);
factor(f,s) = sam(f,s);
totfac = 110;
demand(o,h) = sam(o,h);
totdem(h) = sum(o, demand(o,h))+sam("tha",h)+sam("thi",h);
endow(f,h) = sam(h,f);
govdem(o) = sam(o,"gov");
supply(s,o) = sam(s,o) + sam("tya",o)$sameas(s,"s_agr")
              + sam("tyi",o)$sameas(s,"s_ind");
t_p(o) = 0;
t_pf("lab",s) = sam("tfl",s) / factor("lab",s);
t_pf("cap",s) = sam("tfk",s) / factor("cap",s);
t_p1("agr",h) = sam("tha",h) / sam("agr",h);
t_p1("ind",h) = sam("thi",h) / sam("ind",h);
t_p2(o,h) = 0;
tp_pf(f,s) = 1 + t_pf(f,s);
tp_p(o,h) = 1 + t_p1(o,h) + t_p2(o,h);

$if not exist demo.har $include gdxtohar

$ontext
$model:demo

$sectors:
  y(s)              ! production
  c(h)              ! aggregate consumption
  va(s)             ! value added
  g                 ! government expenditure

$commodities:
  p(o)              ! price of commodity
  pc(h)             ! price of aggregate consumption
  pg                ! price of a government output unit
  pva(s)            ! price index for value added
  pf(f)             ! price of primary factor

$consumers:
  ra(h)             ! representative agent income
  gov               ! government income

$prod:y(s)          t:etrn(s)
  o:p(o)            q:supply(s,o)       p:0.8  a:gov  t:0.2     a:gov  t:t_p(o)
  i:p(o)            q:interm(o,s)
  i:pva(s)          q:totfac

$prod:va(s)         s:esub(s)
  o:pva(s)          q:totfac
  i:pf(f)           q:factor(f,s)       p:tp_pf(f,s)        a:gov     t:t_pf(f,s)

$prod:c(h)          s:1.0
  o:pc(h)           q:totdem(h)
  i:p(o)            q:demand(o,h)       p:tp_p(o,h)   a:gov  t:t_p1(o,h)   a:gov  t:t_p2(o,h)

$prod:g
  o:pg              q:130
  i:p(o)            q:govdem(o)

$demand:ra(h)
  d:pc(h)
  e:pf(f)           q:endow(f,h)

$demand:gov
  d:pg

$report:
  v:r_supply(s,o)   o:p(o)              prod:y(s)
  v:r_cons(o,h)     i:p(o)              prod:c(h)

$offtext
$sysinclude mpsgeset demo

* Benchmark replication

demo.iterlim = 0;
$include demo.gen
solve demo using mcp;
abort$(abs(demo.objval) gt 1e-7) "*** demo does not calibrate ! ***";
demo.iterlim = 1000;

* Fixation of a numeraire to allow comparison

pg.fx = pg.l;

* Counterfactual

t_p(o) = 0.05;
t_pf(f,s) = t_pf(f,s) * 1.05;
t_p2(o,"hh") = 0.05;

$include demo.gen
solve demo using mcp;

* Results

parameters
  res_com           results by commodity in % change from basis
  res_agg           aggregate results in % change from basis; 

res_com("s_agr",o) = (r_supply.l("s_agr",o)/supply("s_agr",o) - 1) * 100;
res_com("s_ind",o) = (r_supply.l("s_ind",o)/supply("s_ind",o) - 1) * 100;
res_com("cons_hh",o) = (r_cons.l(o,"hh")/demand(o,"hh") - 1) * 100;
res_com("cons_fh",o) = (r_cons.l(o,"fh")/demand(o,"fh") - 1) * 100;
res_com("price",o) = (p.l(o) - 1) * 100;

res_agg("c",h) = (c.l(h) - 1 ) * 100;
res_agg("pc",h) = (pc.l(h) - 1 ) * 100;
res_agg("ra_inc",h) = (ra.l(h)/(sum(f, endow(f,h))) - 1 ) * 100;

option decimals = 6;
display res_com, res_agg;
