$title Difftax model

* -----------------------------------------------------------------------------------
* Closed economy with benchmark taxes on input and endogenous taxes on output
* associated with equal yield constraint.
* Laurent Cretegny, CoPS, Monash University, Australia, 2004.
* -----------------------------------------------------------------------------------

$if not exist difftax.gdx $call gdxxrw i=difftax.xls o=difftax.gdx index=declare

sets
  s                 sector
  o                 commodity
  f                 primary factor;

parameters
  sam(*,*)          benchmark social accounting matrix
  supply(s,o)       supply of output [[sppl:s*o]]
  interm(o,s)       intermediate demand [[intr:o*s]]
  factor(f,s)       factor demand [[fctr:f*s]]
  demand(o)         demand for consumption [[dmnd:o]]
  govdem(o)         government demand [[gvdm:o]]
  endow(f)          endowment [[endw:f]]
  etrn(s)           elasticity of transformation [[etrn:s]]
  esub(s)           elasticity of substitution [[esub:s]]
  esubc             elasticity of substitution for consumption [[esbc:]]
  bto(s,o)          tax rate on output [[bto:s*o]]
  btf(f,s)          tax rate on primary factor [[btf:f*s]]
  bpf(f,s)          benchmark price for primary factor [[bpf:f*s]]
  toi               benchmark tax index [[to:]];

$gdxin difftax
$load s o f etrn esub esubc sam
$gdxin

supply(s,o) = sam(s,o);
interm(o,s) = sam(o,s);
factor(f,s) = sam(f,s);
demand(o) = sam(o,"ra");
govdem(o) = sam(o,"gov");
endow(f) = sam("ra",f);
bto(s,o) = 1;
toi = 0;
btf("cap",s)$factor("cap",s) = sam("tcap",s) / factor("cap",s);
bpf(f,s) = 1 + btf(f,s);

* $if not exist difftax.har $include gdxtohar

$ontext
$model:difftax

$sectors:
  y(s)              ! production
  u                 ! utility index
  g                 ! government expenditure

$commodities:
  pd(o)             ! domestic price of commodity
  pf(f)             ! price of primary factor
  pu                ! price index for utility
  pg                ! price of a government output unit

$consumers:
  ra                ! representative agent income
  gov               ! government income

$auxiliary:
  to                ! endogenous tax index associated with equal yield constraint

$prod:y(s)          t:etrn(s)           va:esub(s)
  o:pd(o)           q:supply(s,o)       a:gov          n:to      m:bto(s,o)
  i:pd(o)           q:interm(o,s)
  i:pf(f)           q:factor(f,s)       p:bpf(f,s)     a:gov     t:btf(f,s)     va:

$prod:u             s:esubc
  o:pu              q:(sum(o, demand(o)))
  i:pd(o)           q:demand(o)

$prod:g
  o:pg              q:(sum(o, govdem(o)))
  i:pd(o)           q:govdem(o)

$demand:ra
  d:pu
  e:pf(f)           q:endow(f)

$demand:gov
  d:pg

$constraint:to
* Hold real government consumption fixed
* In the levels, real govt consumption is GOV/PG=SUM(o,DEMAND(o))
  gov/pg =e= sum(o, govdem(o));

$report:
  v:rsupply(s,o)    o:pd(o)             prod:y(s)
  v:rdemand(o)      i:pd(o)             prod:u

$offtext
$sysinclude mpsgeset difftax

* Benchmark replication

difftax.iterlim = 0;
$include difftax.gen
solve difftax using mcp;
abort$(abs(difftax.objval) gt 1e-7) "*** difftax does not calibrate ! ***";
difftax.iterlim = 1000;

* Initialize levels on auxiliary variables

to.lo = -inf;
to.l = toi;

* Fixation of a numeraire to allow comparison

pu.fx = pu.l;

* Counterfactual : tax on output only

btf("cap","sind") = 0;

$include difftax.gen
solve difftax using mcp;

* Results

parameters
  res_com           results by commodity in % change from basis
  res_agg           aggregate results in % change from basis; 

res_com("sagr",o) = (rsupply.l("sagr",o)/supply("sagr",o) - 1) * 100;
res_com("sind",o) = (rsupply.l("sind",o)/supply("sind",o) - 1) * 100;
res_com("demand",o) = (rdemand.l(o)/demand(o) - 1) * 100;
res_com("price",o) = (pd.l(o) - 1) * 100;

res_agg("u") = (u.l - 1) * 100;
res_agg("pu") = (pu.l - 1) * 100;
res_agg("ra_inc") = (ra.l/(sum(o, demand(o))) - 1) * 100;
res_agg("to") = to.l * 100;

option decimals = 6;
display res_com, res_agg;
