$title Employ model

* ----------------------------------------------------------------------------
* Closed economy with lower bound on the real wage and classical unemployment.
* Laurent Cretegny, CoPS, Monash University, Australia, 2004.
* ----------------------------------------------------------------------------

$if not exist employ.gdx $call gdxxrw i=employ.xls o=employ.gdx index=declare

sets
  s                 sector
  o                 commodity
  f                 primary factor;

alias (o,o1);

parameters
  sam(*,*)          benchmark social accounting matrix
  supply(s,o)       supply of output [[sppl:s*o]]
  interm(o,s)       intermediate demand [[intr:o*s]]
  factor(f,s)       factor demand [[fctr:f*s]]
  demand(o)         demand for consumption [[dmnd:o]]
  govdem(o)         government demand [[gvdm:o]]
  endow(f)          endowment [[endw:f]]
  endl              labour endowment [[endl:]]
  endc              capital endowment [[endc:]]
  etrn(s)           elasticity of transformation [[etrn:s]]
  esub(s)           elasticity of substitution [[esub:s]]
  esubc             elasticity of substitution for consumption [[esbc:]]
  bto(s,o)          tax rate on output [[bto:s*o]]
  btf(f,s)          tax rate on primary factor [[btf:f*s]]
  bpf(f,s)          benchmark price for primary factor [[bpf:f*s]]
  epli              benchmark employment index [[epl:]]
  upli              benchmark unemployment index [[upl:]];

$gdxin employ
$load s o f etrn esub esubc sam
$gdxin

supply(s,o) = sam(s,o);
interm(o,s) = sam(o,s);
factor(f,s) = sam(f,s);
demand(o) = sam(o,"ra");
govdem(o) = sam(o,"gov");
endow(f) = sam("ra",f);
endl = sam("ra","lab");
endc = sam("ra","cap");
bto(s,o) = 0;
btf("cap",s)$factor("cap",s) = sam("tcap",s) / factor("cap",s);
bpf(f,s) = 1 + btf(f,s);
epli = 1;
upli = 0;

$if not exist employ.har $include gdxtohar

$ontext
$model:employ

$sectors:
  y(s)              ! production
  va(s)             ! value-added
  u                 ! utility index
  g                 ! government expenditure

$commodities:
  pd(o)             ! domestic price of commodity
  pf(f)             ! price of primary factor
  pva(s)            ! price of value-added
  pu                ! price index for utility
  pg                ! price of a government output unit

$consumers:
  ra                ! representative agent income
  gov               ! government income

$auxiliary:
  epl               ! employment index
*  upl               ! unemployment index

$prod:y(s)          t:etrn(s)           s:0
  o:pd(o)           q:supply(s,o)       a:gov          t:bto(s,o)
  i:pd(o)           q:interm(o,s)
  i:pva(s)          q:(sum(f, bpf(f,s)*factor(f,s)))

$prod:va(s)         s:esub(s)
  o:pva(s)          q:(sum(f, bpf(f,s)*factor(f,s)))
  i:pf(f)           q:factor(f,s)       p:bpf(f,s)     a:gov     t:btf(f,s)

$prod:u             s:esubc
  o:pu              q:(sum(o, demand(o)))
  i:pd(o)           q:demand(o)

$prod:g
  o:pg              q:(sum(o, govdem(o)))
  i:pd(o)           q:govdem(o)

$demand:ra
  d:pu
  e:pf("lab")       q:endl                        r:epl
*  e:pf("lab")       q:endow("lab")
*  e:pf("lab")       q:-endow("lab")               r:upl
  e:pf("cap")       q:endc

$demand:gov
  d:pg

$constraint:epl
  pf("lab") =g= pu;

$report:
  v:rsupply(s,o)    o:pd(o)             prod:y(s)
  v:rinterm(o,s)    i:pd(o)             prod:y(s)
  v:rdemand(o)      i:pd(o)             prod:u

$offtext
$sysinclude mpsgeset employ

* Initial levels on auxiliary variables

epl.l = epli;
*upl.l = upli;

* Benchmark replication

employ.iterlim = 0;
$include employ.gen
solve employ using mcp;
abort$(abs(employ.objval) gt 1e-7) "*** employ does not calibrate ! ***";
employ.iterlim = 1000;

* Fixation of a numeraire to allow comparison

pu.fx = pu.l;

* Counterfactual : introduction of a tax on output

bto("sind",o) = 0.05;

$include employ.gen
solve employ using mcp;

* Results

parameters
  res_com           results by commodity in % change from basis
  res_agg           aggregate results in % change from basis; 

res_com("sagr",o) = (rsupply.l("sagr",o)/supply("sagr",o) - 1) * 100;
res_com("sind",o) = (rsupply.l("sind",o)/supply("sind",o) - 1) * 100;
res_com("demand",o) = (rdemand.l(o)/demand(o) - 1) * 100;
res_com("price",o) = (pd.l(o) - 1) * 100;

res_agg("u") = (u.l - 1) * 100;
res_agg("pu") = (pu.l - 1) * 100;
res_agg("ra_inc") = (ra.l/(sum(o, demand(o))) - 1) * 100;

* Values for complementarity implementation

parameters
  rev(s)
  cost(s)
  chksup(s,o)
  chkint(o,s);

rev(s) = sum(o, supply(s,o)/sum(o1, supply(s,o1))*(pd.l(o)*(1-bto(s,o)))
         **(1+etrn(s)))**(1/(1+etrn(s)));
cost(s) = (sum(o, interm(o,s)/(sum(o1, interm(o1,s))+sum(f, bpf(f,s)*factor(f,s)))*pd.l(o)
          **(1-0))+sum(f, bpf(f,s)*factor(f,s))/(sum(o1, interm(o1,s))+sum(f, bpf(f,s)
          *factor(f,s)))*pva.l(s)**(1-0))**(1/(1-0));
chksup(s,o) = supply(s,o)*(pd.l(o)*(1-bto(s,o))/rev(s))**etrn(s) * y.l(s);
chkint(o,s) = interm(o,s)*(cost(s)/pd.l(o))**0 * y.l(s);

option decimals = 6;
display res_com, res_agg, rev, cost, chksup, chkint;
