$title	Read GTAP6 Basedata and Replicate the Benchmark in MPSGE

$if not set ds $set ds china

*	Extract files from the dataset, retrieve the GDX file
*	and then cleanup:

$call 'gmsunzip %ds%.zip'
$call 'har2gdx basedata.har'
$call 'har2gdx default.prm'
$call 'gmszip -f -m %ds%'

set	i(*)	Goods
	f(*)	Factors
	r(*)	Regions 

$gdxin 'basedata.gdx'
$load r=reg f=endw_comm i=prod_comm

set	cgd(i)	Investment good /cgds/,
	rnum(r)	Numeraire region,
	src	Sources /domestic, imported/;

alias (r,s), (i,j);

parameters
	vdga(i,r)	Government - domestic purchases at agents' prices,
	viga(i,r)	Government - imports at agents' prices,
	vdgm(i,r)	Government - domestic purchases at market prices,
	vigm(i,r)	Government - imports at market prices,
	vdpa(i,r)	Private households - domestic purchases at agents' prices,
	vipa(i,r)	Private households - imports at agents' prices,
	vdpm(i,r)	Private households - domestic purchases at market prices,
	vipm(i,r)	Private households - imports at market prices
	evoa(f,r)	Endowments - output at agents' prices,
	evfa(f,j,r)	Endowments - firms' purchases at agents' prices,
	vfm(f,j,r)	Endowments - Firms' purchases at market prices,
	vdfa(i,j,r)	Intermediates - firms' domestic purchases at agents' prices,
	vifa(i,j,r)	Intermediates - Firms' imports at agents' prices,
	vdfm(i,j,r)	Intermediates - firms' domestic purchases at market prices,
	vifm(i,j,r)	Intermediates - firms' imports at market prices,
	vims(i,r,s)	Trade - bilateral imports at market prices,
	viws(i,r,s)	Trade - bilateral imports at world prices,
	vxmd(i,r,s)	Trade - bilateral exports at market prices,
	vxwd(i,r,s)	Trade - bilateral exports at world prices,
	vst(i,r)	Trade - exports for international transportation
	vtwr(i,j,r,s)	Trade - Margins for international transportation at world prices
	ftrv(f,j,r)	Taxes - factor employment tax revenue,
	fbep(f,j,r)	Protection - factor-based subsidies,
	isep(i,j,r,src) Protection - intermediate input subsidies,
	osep(i,r)	Protection - ordinary output subsidies,
	adrv(i,r,s)	Protection - anti-dumping duty
	tfrv(i,r,s)	Protection - ordinary import duty
	purv(i,r,s)	Protection - price undertaking export tax equivalent,
	vrrv(i,r,s)	Protection - VER export tax equivalent
	mfrv(i,r,s)	Protection - MFA export tax equivalent
	xtrv(i,r,s)	Protection - ordinary export tax;

$load vdga viga vdgm vigm vdpa vipa vdpm vipm evoa evfa vfm vdfa 
$load vifa vdfm vifm vims viws vxmd vxwd vst vtwr ftrv fbep isep 
$load osep adrv tfrv purv vrrv mfrv xtrv 

*	Declare some intermediate arrays which are required to 
*	evaluate tax rates:

parameter	vdm(i,r)	Aggregate demand for domestic output,
		vom(i,r)	Total supply at market prices,
		voa(i,r)	Output at producer prices;

vdm(i,r) = vdpm(i,r) + vdgm(i,r) + sum{j, vdfm(i,j,r)};
vom(i,r) = vdm(i,r) + sum(s, vxmd(i,r,s)) + vst(i,r);
voa(j,r) = sum(f, evfa(f,j,r)) + sum(i, vdfa(i,j,r) + vifa(i,j,r));
vom(cgd,r) = voa(cgd,r);

parameter
	rto(i,r)	Output (or income) subsidy rates
	rtf(f,j,r)	Primary factor and commodity rates taxes 
	rtpd(i,r)	Private domestic consumption taxes
	rtpi(i,r)	Private import consumption tax rates
	rtgd(i,r)	Government domestic rates
	rtgi(i,r)	Government import tax rates
	rtfd(i,j,r)	Firms domestic tax rates
	rtfi(i,j,r)	Firms' import tax rates
	rtxs(i,r,s)	Export subsidy rates
	rtms(i,r,s)	Import taxes rates;

rto(i,r) = 1 - voa(i,r)/vom(i,r);
rtf(f,j,r)$vfm(f,j,r)  = evfa(f,j,r)/vfm(f,j,r) - 1;
rtpd(i,r)$vdpm(i,r)   = vdpa(i,r)/vdpm(i,r) - 1;
rtpi(i,r)$vipm(i,r)   = vipa(i,r)/vipm(i,r) - 1;
rtgd(i,r)$vdgm(i,r)   = vdga(i,r)/vdgm(i,r) - 1;
rtgi(i,r)$vigm(i,r)   = viga(i,r)/vigm(i,r) - 1;
rtfd(i,j,r)$vdfm(i,j,r) = vdfa(i,j,r)/vdfm(i,j,r) - 1;
rtfi(i,j,r)$vifm(i,j,r) = vifa(i,j,r)/vifm(i,j,r) - 1;
rtxs(i,r,s)$vxwd(i,r,s) = 1-vxwd(i,r,s)/vxmd(i,r,s);
rtms(i,r,s)$viws(i,r,s) = vims(i,r,s)/viws(i,r,s) - 1;

*	Store benchmark tax rates to construct reference prices:

parameter
	rtf0(f,j,r)	Primary factor and commodity rates taxes 
	rtpd0(i,r)	Private domestic consumption taxes
	rtpi0(i,r)	Private import consumption tax rates
	rtgd0(i,r)	Government domestic rates
	rtgi0(i,r)	Government import tax rates
	rtfd0(i,j,r)	Firms domestic tax rates
	rtfi0(i,j,r)	Firms' import tax rates
	rtxs0(i,r,s)	Export subsidy rates
	rtms0(i,r,s)	Import taxes rates;

rtf0(f,j,r) = rtf(f,j,r);
rtpd0(i,r) = rtpd(i,r);
rtpi0(i,r) = rtpi(i,r);
rtgd0(i,r) = rtgd(i,r);
rtgi0(i,r) = rtgi(i,r);
rtfd0(i,j,r) = rtfd(i,j,r);
rtfi0(i,j,r) = rtfi(i,j,r);
rtxs0(i,r,s) = rtxs(i,r,s);
rtms0(i,r,s) = rtms(i,r,s);

parameter	pvxmd(i,s,r)	Import price (power of benchmark tariff)
		pvtwr(i,s,r)	Import price for transport services;

pvxmd(i,s,r) = (1+rtms0(i,s,r)) * (1-rtxs0(i,s,r));
pvtwr(i,s,r) = 1+rtms0(i,s,r);

parameter	
	vtw(j)		Aggregate international transportation services,
	vpm(r)		Aggregate private demand,
	vgm(r)		Aggregate public demand,
	vim(i,r)	Aggregate imports,
	evom(f,r)	Aggregate factor endowment at market prices,
	vb(*)		Current account balance;

vtw(j) = sum(r, vst(j,r));
vpm(r) = sum(i, vdpm(i,r)*(1+rtpd0(i,r)) + vipm(i,r)*(1+rtpi0(i,r)));
vgm(r) = sum(i, vdgm(i,r)*(1+rtgd0(i,r)) + vigm(i,r)*(1+rtgi0(i,r)));
vim(i,r) = vipm(i,r) + vigm(i,r) + sum(j, vifm(i,j,r));
evom(f,r) = sum(j, vfm(f,j,r));
vb(r) = vpm(r) + vgm(r) + sum(cgd,vom(cgd,r)) 
	- sum(f, evom(f,r))
	- sum(j,  vom(j,r)*rto(j,r))
	- sum(j,  sum(i, vdfm(i,j,r)*rtfd(i,j,r) + vifm(i,j,r)*rtfi(i,j,r)))
	- sum(j,  sum(f, vfm(f,j,r)*rtf(f,j,r)))
	- sum(i, vdpm(i,r)*rtpd(i,r) + vipm(i,r)*rtpi(i,r))
	- sum(i, vdgm(i,r)*rtgd(i,r) + vigm(i,r)*rtgi(i,r))
	- sum((i,s), rtms(i,s,r) *  (vxmd(i,s,r) * (1-rtxs(i,s,r)) + sum(j,vtwr(j,i,s,r))))
	+ sum((i,s), rtxs(i,r,s) * vxmd(i,r,s));

vb("chksum") = sum(r, vb(r));
display vb;

parameter
	esubd(i)	Elasticity of substitution (M versus D),
	esubva(j)	Elasticity of substitution between factors
	esubm(i)	Intra-import elasticity of substitution;

$gdxin 'default.gdx'
$load esubd esubva esubm
$gdxin


rnum(r) = yes$(vpm(r)=smax(s,vpm(s)));
display rnum;

$ontext
$model:gtap6

$sectors:
	c(r)			! Consumption
	g(r)			! Government demand
	y(i,r)$vom(i,r)		! Supply
	m(i,r)$vim(i,r)		! Imports
	yt(j)$vtw(j)		! Transportation services

$commodities:
	pc(r)			! Private consumption price index
	pg(r)			! Public consumption price index
	py(j,r)$vom(j,r)	! Domestic output price
	pm(j,r)$vim(j,r)	! Import price
	pt(j)$vtw(j)		! Transportation services
	pf(f,r)$evoa(f,r)	! Primary factors

$consumers:
	ra(r)			! Representative agent

$prod:y(j,r)$vom(j,r)	s:0    i.tl:esubd(i)  va:esubva(j)
	o:py(j,r)	q:vom(j,r)	a:ra(r)  t:rto(j,r)
	i:py(i,r)	q:vdfm(i,j,r)	p:(1+rtfd0(i,j,r)) i.tl:  a:ra(r) t:rtfd(i,j,r)
	i:pm(i,r)	q:vifm(i,j,r)	p:(1+rtfi0(i,j,r)) i.tl:  a:ra(r) t:rtfi(i,j,r)
	i:pf(f,r)	q:vfm(f,j,r)	p:(1+rtf0(f,j,r))  va:    a:ra(r) t:rtf(f,j,r)

$prod:yt(j)$vtw(j)  s:1
	o:pt(j)		q:vtw(j)
	i:py(j,r)	q:vst(j,r)

$prod:c(r)  s:1  i.tl:esubd(i)
	o:pc(r)		q:vpm(r)
	i:py(i,r)	q:vdpm(i,r)	i.tl: p:(1+rtpd0(i,r)) a:ra(r) t:rtpd(i,r)
	i:pm(i,r)	q:vipm(i,r)	i.tl: p:(1+rtpi0(i,r)) a:ra(r) t:rtpi(i,r)

$prod:g(r)  s:1  i.tl:esubd(i)
	o:pg(r)		q:vgm(r)
	i:py(i,r)	q:vdgm(i,r)	i.tl: p:(1+rtgd0(i,r)) a:ra(r) t:rtgd(i,r)
	i:pm(i,r)	q:vigm(i,r)	i.tl: p:(1+rtgi0(i,r)) a:ra(r) t:rtgi(i,r)

$prod:m(i,r)$vim(i,r)	s:esubm(i)  s.tl:0
	o:pm(i,r)	q:vim(i,r)
	i:py(i,s)	q:vxmd(i,s,r)	p:pvxmd(i,s,r) s.tl:
+		a:ra(s) t:(-rtxs(i,s,r))
+		a:ra(r) t:(rtms(i,s,r)*(1-rtxs(i,s,r)))
	i:pt(j)#(s)	q:vtwr(j,i,s,r) p:pvtwr(i,s,r) s.tl:
+		a:ra(r) t:rtms(i,s,r)

$demand:ra(r)
	d:pc(r)		q:vpm(r)
	e:pc(rnum)	q:vb(r)
	e:pg(r)		q:(-vgm(r))
	e:py(cgd,r)	q:(-vom(cgd,r))
	e:pf(f,r)	q:evom(f,r)

$offtext
$sysinclude mpsgeset gtap6

gtap6.iterlim = 0;
$include gtap6.gen
solve gtap6 using mcp;
